// src/lib/apiClient.ts
import { error } from '@sveltejs/kit'; // SvelteKit's error helper

// --- Configuration ---
// Get the base URL from environment variables provided by Vite/SvelteKit
// Ensure VITE_API_BASE_URL is set in your .env file (e.g., VITE_API_BASE_URL=http://localhost:8000/api)
const BASE_URL = import.meta.env.VITE_API_BASE_URL;

if (!BASE_URL) {
    console.error('VITE_API_BASE_URL is not defined. Please set it in your .env file.');
    // In a real app, you might throw an error here or have a default,
    // but logging is often sufficient during development.
}

export class ApiClientError extends Error {
    status: number;
    errorData: unknown;

    constructor(message: string, status: number, errorData: unknown = null) {
        super(message);
        this.name = 'ApiClientError';
        this.status = status;
        this.errorData = errorData;

        // --- Corrected Conditional Check ---
        // Check if the static method exists on the Error constructor object
        if (typeof (Error as any).captureStackTrace === 'function') {
            // Call it if it exists, casting Error to 'any' to bypass static type check
            (Error as any).captureStackTrace(this, ApiClientError);
        }
        // else {
        // Optional: Fallback if captureStackTrace is not available
        // You might assign the stack from a new error instance,
        // though `super(message)` often handles basic stack creation.
        // this.stack = new Error(message).stack;
        // }
        // --- End Corrected Check ---
    }
}

// --- Core Fetch Function ---
interface RequestOptions extends Omit<RequestInit, 'body'> {
    // Can add custom options here if needed later
}

async function request<T = unknown>( // Generic type T for expected response data
    method: string,
    path: string, // Relative path (e.g., /v1/health)
    data?: unknown, // Optional request body data
    options: RequestOptions = {} // Optional fetch options (headers, etc.)
): Promise<T> {

    if (!BASE_URL) {
        // Or use SvelteKit's error helper for server-side/universal loads
        // error(500, 'API Base URL is not configured.');
        throw new Error('API Base URL (VITE_API_BASE_URL) is not configured.');
    }

    // Construct the full URL, handling potential leading/trailing slashes
    const cleanBase = BASE_URL.replace(/\/$/, ''); // Remove trailing slash from base
    const cleanPath = path.replace(/^\//, ''); // Remove leading slash from path
    const url = `${cleanBase}/${cleanPath}`;

    // Default headers
    const headers = new Headers({
        Accept: 'application/json',
        ...options.headers // Spread custom headers from options
    });

    // Fetch options
    const fetchOptions: RequestInit = {
        method: method.toUpperCase(),
        headers,
        ...options // Spread other custom options (credentials, mode, cache, etc.)
    };

    // Add body and Content-Type header if data is provided
    if (data !== undefined && data !== null) {
        headers.set('Content-Type', 'application/json');
        fetchOptions.body = JSON.stringify(data);
    }

    // Add credentials option if needed for cookies/auth later
    // fetchOptions.credentials = 'include';

    try {
        const response = await fetch(url, fetchOptions);

        // Check if the response is successful (status code 200-299)
        if (!response.ok) {
            let errorJson: unknown = null;
            try {
                // Try to parse error details from the response body
                errorJson = await response.json();
            } catch (e) {
                // Ignore if response body isn't valid JSON
                console.warn('API Error response was not valid JSON.', response.status, response.statusText)
            }
            // Throw a custom error with status and potentially parsed error data
            throw new ApiClientError(
                `API request failed: ${response.status} ${response.statusText}`,
                response.status,
                errorJson
            );
        }

        // Handle successful responses with no content (e.g., 204 No Content)
        if (response.status === 204) {
            // Type assertion needed because Promise<T> expects a value,
            // but 204 has no body. We return null. Adjust T if needed.
            return null as T;
        }

        // Parse successful JSON response
        const responseData = await response.json();
        return responseData as T; // Assert the type based on the generic T

    } catch (err) {
        // Handle network errors or errors thrown above
        console.error(`API Client request error: ${method} ${path}`, err);

        // Re-throw the error so calling code can handle it
        // If it's already our custom error, re-throw it directly
        if (err instanceof ApiClientError) {
            throw err;
        }
        // Otherwise, wrap network or other errors
        throw new ApiClientError(
            `Network or unexpected error during API request: ${err instanceof Error ? err.message : String(err)}`,
            0, // Use 0 or a specific code for network errors
            err
        );
    }
}

// --- Helper Methods ---

export const apiClient = {
    get: <T = unknown>(path: string, options: RequestOptions = {}): Promise<T> => {
        return request<T>('GET', path, undefined, options);
    },

    post: <T = unknown>(path: string, data: unknown, options: RequestOptions = {}): Promise<T> => {
        return request<T>('POST', path, data, options);
    },

    put: <T = unknown>(path: string, data: unknown, options: RequestOptions = {}): Promise<T> => {
        return request<T>('PUT', path, data, options);
    },

    delete: <T = unknown>(path: string, options: RequestOptions = {}): Promise<T> => {
        // Note: DELETE requests might have a body, but often don't. Adjust if needed.
        return request<T>('DELETE', path, undefined, options);
    },

    patch: <T = unknown>(path: string, data: unknown, options: RequestOptions = {}): Promise<T> => {
        return request<T>('PATCH', path, data, options);
    }
    // Can add other methods (HEAD, OPTIONS) if necessary
};

// Default export can sometimes be convenient, but named export is clear
// export default apiClient;